@@ -59,6 +59,7 @@
 #define I_KNOW_THE_PACKAGEKIT_GLIB2_API_IS_SUBJECT_TO_CHANGE
 #include <packagekit-glib2/packagekit.h>
 
+#include "pk-backend-zypp-private.h"
 #include "zypp-utils.h"
 
 gchar * _repoName;
@@ -68,6 +69,8 @@ using namespace std;
 using namespace zypp;
 using zypp::filesystem::PathInfo;
 
+extern PkBackendZYppPrivate *priv;
+
 /**
  * Initialize Zypp (Factory method)
  */
@@ -192,8 +195,11 @@ zypp_is_valid_repo (PkBackend *backend, RepoInfo repo)
 ResPool
 zypp_build_pool (PkBackend *backend, gboolean include_local)
 {
+	static gboolean repos_loaded = FALSE;
+
 	ZYpp::Ptr zypp = get_zypp (backend);
 
+	// the target is loaded or unloaded on request
 	if (include_local) {
 		// FIXME have to wait for fix in zypp (repeated loading of target)
 		if (sat::Pool::instance().reposFind( sat::Pool::systemRepoAlias() ).solvablesEmpty ())
@@ -211,12 +217,14 @@ zypp_build_pool (PkBackend *backend, gboolean include_local)
 		}
 	}
 
+	// we only load repositories once.
+	if (repos_loaded)
+		return zypp->pool();
+
 	// Add resolvables from enabled repos
 	RepoManager manager;
-	list<RepoInfo> repos;
 	try {
-		repos = list<RepoInfo>(manager.repoBegin(),manager.repoEnd());
-		for (list<RepoInfo>::iterator it = repos.begin(); it != repos.end (); it++) {
+		for (RepoManager::RepoConstIterator it = manager.repoBegin(); it != manager.repoEnd(); it++) {
 			RepoInfo repo (*it);
 
 			// skip disabled repos
@@ -230,7 +238,9 @@ zypp_build_pool (PkBackend *backend, gboolean include_local)
                         //FIXME see above, skip already cached repos
                         if (sat::Pool::instance().reposFind( repo.alias ()) == Repository::noRepository)
                                 manager.loadFromCache (repo);
+
 		}
+		repos_loaded = true;
 	} catch (const repo::RepoNoAliasException &ex) {
                 g_error ("Can't figure an alias to look in cache");
         } catch (const repo::RepoNotCachedException &ex) {
@@ -489,7 +499,7 @@ zypp_signature_required (PkBackend *backend, const PublicKey &key)
 {
 	gboolean ok = FALSE;
 
-	if (find (_signatures[backend]->begin (), _signatures[backend]->end (), key.id ()) == _signatures[backend]->end ()) {
+	if (find (priv->signatures[backend]->begin (), priv->signatures[backend]->end (), key.id ()) == priv->signatures[backend]->end ()) {
 		RepoInfo info = zypp_get_Repository (backend, _repoName);
 		if (info.type () == repo::RepoType::NONE)
 			pk_backend_error_code (backend, PK_ERROR_ENUM_INTERNAL_ERROR,
@@ -519,7 +529,7 @@ zypp_signature_required (PkBackend *backend, const string &file, const string &i
 {
         gboolean ok = FALSE;
 
-	if (find (_signatures[backend]->begin (), _signatures[backend]->end (), id) == _signatures[backend]->end ()) {
+	if (find (priv->signatures[backend]->begin (), priv->signatures[backend]->end (), id) == priv->signatures[backend]->end ()) {
 		RepoInfo info = zypp_get_Repository (backend, _repoName);
 		if (info.type () == repo::RepoType::NONE)
 			pk_backend_error_code (backend, PK_ERROR_ENUM_INTERNAL_ERROR,
@@ -549,7 +559,7 @@ zypp_signature_required (PkBackend *backend, const string &file)
 {
 	gboolean ok = FALSE;
 
-	if (find (_signatures[backend]->begin (), _signatures[backend]->end (), file) == _signatures[backend]->end ()) {
+	if (find (priv->signatures[backend]->begin (), priv->signatures[backend]->end (), file) == priv->signatures[backend]->end ()) {
         	RepoInfo info = zypp_get_Repository (backend, _repoName);
 		if (info.type () == repo::RepoType::NONE)
 			pk_backend_error_code (backend, PK_ERROR_ENUM_INTERNAL_ERROR,
@@ -730,8 +740,8 @@ zypp_get_patches (PkBackend *backend, set<PoolItem> &patches)
 
         for (ResPoolProxy::const_iterator it = zypp->poolProxy ().byKindBegin<Patch>();
                         it != zypp->poolProxy ().byKindEnd<Patch>(); it ++) {
-                // check if patch is needed
-                if((*it)->isNeeded()) {
+                // check if the patch is needed and not set to taboo
+                if((*it)->isNeeded() && !((*it)->candidateObj ().isUnwanted())) {
 			Patch::constPtr patch = asKind<Patch>((*it)->candidateObj ().resolvable ());
 			if (_updating_self) {
 				if (patch->restartSuggested ())
